#include "camera.h"
#include "math.h"
#include <stdio.h>
//#include "windows.h"

#define SQR(x) (x*x)



GLfloat length( _3DVector * v)
{
	return (GLfloat)(sqrt(SQR(v->x)+SQR(v->y)+SQR(v->z)));
}

_3DVector normalize( _3DVector v)
{
	_3DVector res;
	float l = length(&v);
	if (l == 0.0f) 
	{
		res.x = 0.0f;
		res.y = 0.0f;
		res.z = 0.0f;
	}
	else
	{
		res.x = v.x / l;
		res.y = v.y / l;
		res.z = v.z / l;
	}
	return res;
}

_3DVector add (_3DVector v, _3DVector u)
{
	_3DVector res;
	res.x = v.x+u.x;
	res.y = v.y+u.y;
	res.z = v.z+u.z;
	return res;
}

_3DVector subtract (_3DVector v, _3DVector u)
{
	_3DVector res;
	res.x = v.x-u.x;
	res.y = v.y-u.y;
	res.z = v.z-u.z;
	return res;
}


_3DVector product (_3DVector v, float r)
{
	_3DVector res;
	res.x = v.x*r;
	res.y = v.y*r;
	res.z = v.z*r;
	return res;
}

_3DVector cross (_3DVector * u, _3DVector * v)
{
	_3DVector resVector;
	resVector.x = u->y*v->z - u->z*v->y;
	resVector.y = u->z*v->x - u->x*v->z;
	resVector.z = u->x*v->y - u->y*v->x;
	return resVector;
}

float dot (_3DVector v, _3DVector u)	//dot product
{
	return v.x*u.x+v.y*u.y+v.z*u.z;
}




//-------------------------------------------------------------------------------------------
void init(Camera *c)
{
	//Init with standard OGL values:
	c->Position.x= 0.0; c->Position.y = 0.0; c->Position.z = 0.0;
	c->ViewDir.x= 0.0; c->ViewDir.y = 0.0; c->ViewDir.z = -1.0;
	c->RightVector.x= 1.0; c->RightVector.y = 0.0; c->RightVector.z = 0.0;
	c->UpVector.x= 0.0; c->UpVector.y = 1.0; c->UpVector.z = 0.0;

	//Only to be sure:
	c->RotatedX = c->RotatedY = c->RotatedZ = 0.0;
}
//-------------------------------------------------------------------------------------------

void Move (Camera *c, _3DVector Direction)
{
	c->Position = add(c->Position,Direction);
}
//-------------------------------------------------------------------------------------------

void RotateX (Camera *c, GLfloat Angle)
{
	c->RotatedX += Angle;
	
	//Rotate viewdir around the right vector:
	c->ViewDir = normalize(add(product(c->ViewDir,cos(Angle*PIdiv180)),
						       product(c->UpVector,sin(Angle*PIdiv180))));

	//now compute the new UpVector (by cross product)
	c->UpVector = product(cross(&(c->ViewDir), &(c->RightVector)),-1);

	
}
//-------------------------------------------------------------------------------------------

void RotateY (Camera *c, GLfloat Angle)
{
	c->RotatedY += Angle;
	
	//Rotate viewdir around the up vector:
	c->ViewDir = normalize(subtract(product(c->ViewDir,cos(Angle*PIdiv180)),
								 product(c->RightVector,sin(Angle*PIdiv180))));

	//now compute the new RightVector (by cross product)
	c->RightVector = cross(&(c->ViewDir), &(c->UpVector));
}
//-------------------------------------------------------------------------------------------

void RotateZ (Camera *c, GLfloat Angle)
{
	c->RotatedZ += Angle;
	
	//Rotate viewdir around the right vector:
	c->RightVector = normalize(add(product(c->RightVector,cos(Angle*PIdiv180)),
								product(c->UpVector,sin(Angle*PIdiv180))));

	//now compute the new UpVector (by cross product)
	c->UpVector = product(cross(&(c->ViewDir), &(c->RightVector)),-1);
}
//-------------------------------------------------------------------------------------------

void Render( Camera *c ) 
{

	//The point at which the camera looks:
	_3DVector ViewPoint = add(c->Position,c->ViewDir);

	//as we know the up vector, we can easily use gluLookAt:
	gluLookAt(	c->Position.x,c->Position.y,c->Position.z,
				ViewPoint.x,ViewPoint.y,ViewPoint.z,
				c->UpVector.x,c->UpVector.y,c->UpVector.z);

}
//-------------------------------------------------------------------------------------------

void MoveForward( Camera *c, GLfloat Distance )
{
	c->Position = add(c->Position,product(c->ViewDir,-Distance));
}
//-------------------------------------------------------------------------------------------

void StrafeRight ( Camera *c, GLfloat Distance )
{
	c->Position = add(c->Position, product(c->RightVector,Distance));
}
//-------------------------------------------------------------------------------------------

void MoveUpward( Camera *c, GLfloat Distance )
{
	c->Position = add(c->Position, product(c->UpVector,Distance));
}
//------------------------------------------------------------------------------------------- end of file
