/**********************************************************************
 
 Camera with OpenGL
 
 March, 13th, 2003
 
 This tutorial was written by Philipp Crocoll
 Contact: 
 philipp.crocoll@web.de
 www.codecolony.de
 
 Every comment would be appreciated.
 
 If you want to use parts of any code of mine:
 let me know and
 use it!
 
 **********************************************************************
 ESC: exit
 
 CAMERA movement:
 w : forwards
 s : backwards
 a : turn left
 d : turn right
 x : turn up
 y : turn down
 v : strafe right
 c : strafe left
 r : move up
 f : move down
 m/n : roll
 
 ***********************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <GLUT/glut.h>
//#include <windows.h>
#include "camera.h"


#define ShowUpvector

Camera camera;


void DrawNet(GLfloat size, GLint LinesX, GLint LinesZ)
{
	glBegin(GL_LINES);
	for (int xc = 0; xc < LinesX; xc++)
	{
		glVertex3f(	-size / 2.0 + xc / (GLfloat)(LinesX-1)*size,
				   0.0,
				   size / 2.0);
		glVertex3f(	-size / 2.0 + xc / (GLfloat)(LinesX-1)*size,
				   0.0,
				   size / -2.0);
	}
	for (int zc = 0; zc < LinesX; zc++)
	{
		glVertex3f(	size / 2.0,
				   0.0,
				   -size / 2.0 + zc / (GLfloat)(LinesZ-1)*size);
		glVertex3f(	size / -2.0,
				   0.0,
				   -size / 2.0 + zc / (GLfloat)(LinesZ-1)*size);
	}
	glEnd();
}

void reshape(int x, int y)
{
	if (y == 0 || x == 0) return;  //Nothing is visible then, so return
	
	//Set a new projection matrix
	glMatrixMode(GL_PROJECTION);  
	glLoadIdentity();
	//Angle of view:40 degrees
	//Near clipping plane distance: 0.5
	//Far clipping plane distance: 20.0
	gluPerspective(40.0,(GLdouble)x/(GLdouble)y,0.5,20.0);
	
	glMatrixMode(GL_MODELVIEW);
	glViewport(0,0,x,y);  //Use the whole window for rendering
}

void Display(void)
{
	glClear(GL_COLOR_BUFFER_BIT);
	glLoadIdentity();
	
	Render(&camera);
	
	//Draw the "world":
	glTranslatef(0.0,-0.5,-6.0);
	
	glScalef(3.0,1.0,3.0);
	
	GLfloat size = 2.0;
	GLint LinesX = 30;
	GLint LinesZ = 30;
	
	GLfloat halfsize = size / 2.0;
	glColor3f(1.0,1.0,1.0);
	glPushMatrix();
	glTranslatef(0.0,-halfsize ,0.0);
	DrawNet(size,LinesX,LinesZ);
	glTranslatef(0.0,size,0.0);
	DrawNet(size,LinesX,LinesZ);
	glPopMatrix();
	glColor3f(0.0,0.0,1.0);
	glPushMatrix();
	glTranslatef(-halfsize,0.0,0.0);	
	glRotatef(90.0,0.0,0.0,halfsize);
	DrawNet(size,LinesX,LinesZ);
	glTranslatef(0.0,-size,0.0);
	DrawNet(size,LinesX,LinesZ);
	glPopMatrix();
	glColor3f(1.0,0.0,0.0);
	glPushMatrix();
	glTranslatef(0.0,0.0,-halfsize);	
	glRotatef(90.0,halfsize,0.0,0.0);
	DrawNet(size,LinesX,LinesZ);
	glTranslatef(0.0,size,0.0);
	DrawNet(size,LinesX,LinesZ);
	glPopMatrix();
	
	
	//finish rendering:
	glFlush();  
	glutSwapBuffers();
	
}
void KeyDown(unsigned char key, int x, int y)
{
	switch (key) 
	{
		case 27:		//ESC
			exit(0);
			break;
		case 'a':		
			RotateY(&camera, 5.0);
			Display();
			break;
		case 'd':		
			RotateY(&camera, -5.0);
			Display();
			break;
		case 'w':		
			MoveForward(&camera, -0.1 ) ;
			Display();
			break;
		case 's':		
			MoveForward(&camera,  0.1 ) ;
			Display();
			break;
		case 'x':		
			RotateX(&camera, 5.0);
			Display();
			break;
		case 'y':		
			RotateX(&camera, -5.0);
			Display();
			break;
		case 'c':		
			StrafeRight(&camera, -0.1);
			Display();
			break;
		case 'v':		
			StrafeRight(&camera, 0.1);
			Display();
			break;
		case 'f':
			MoveUpward(&camera, -0.3);
			Display();
			break;
		case 'r':
			MoveUpward(&camera, 0.3);
			Display();
			break;
			
		case 'm':
			RotateZ(&camera, -5.0);
			Display();
			break;
		case 'n':
			RotateZ(&camera, 5.0);
			Display();
			break;
			
	}
}

int main(int argc, char **argv)
{
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);
	glutInitWindowSize(500,300);
	glutCreateWindow("Camera");
	
	init(&camera);
	_3DVector vec;
	vec.x = 0.0; vec.y = 0.0; vec.z = 3.0;
	
	Move(&camera,vec);
	MoveForward(&camera, 1.0 );
	
	glutDisplayFunc(Display);
	glutReshapeFunc(reshape);
	glutKeyboardFunc(KeyDown);
	
	glutMainLoop();
	return 0;             
}
